'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _toConsumableArray = _interopDefault(require('@babel/runtime/helpers/toConsumableArray'));
var _defineProperty = _interopDefault(require('@babel/runtime/helpers/defineProperty'));
var _slicedToArray = _interopDefault(require('@babel/runtime/helpers/slicedToArray'));
var _typeof = _interopDefault(require('@babel/runtime/helpers/typeof'));
var redux = require('redux');
var reactRedux = require('react-redux');

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }
/**
 * Utils
 */

var ERR = {
  NOT_STRING: function NOT_STRING(name) {
    return "'".concat(name, "' must be a string");
  },
  NOT_ARRAY: function NOT_ARRAY(name) {
    return "'".concat(name, "' must be an array");
  },
  NOT_OBJECT: function NOT_OBJECT(name) {
    return "'".concat(name, "' must be an object");
  },
  NOT_CLASS: function NOT_CLASS(name) {
    return "'".concat(name, "' must be a class");
  },
  NO_SET_STATE: function NO_SET_STATE(name) {
    return "".concat(name, ".setState() is not allowed outside of ").concat(name, " model");
  },
  NO_DISPATCH: function NO_DISPATCH() {
    return "Please don't use 'dispatch' directly in Retalk";
  },
  NOT_EXIST: function NOT_EXIST(name) {
    return "'".concat(name, "' Model dose not exist");
  }
};

var isObject = function isObject(obj) {
  return _typeof(obj) === 'object' && obj !== null && !Array.isArray(obj);
};

var isPromise = function isPromise(obj) {
  return !!obj && typeof obj.then === 'function';
};
/**
 * Add a model, create the model's reducer
 *
 * @param {string} name
 * @param {class} Model
 * @param {object} newModels
 * @param {object} newReducers
 */


var createReducer = function createReducer(name, Model, newModels, newReducers) {
  var model = new Model();
  newModels[name] = model;

  newReducers[name] = function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : model.state || {};
    var action = arguments.length > 1 ? arguments[1] : undefined;

    var _action$type$split = action.type.split('/'),
        _action$type$split2 = _slicedToArray(_action$type$split, 1),
        modelName = _action$type$split2[0];

    if (modelName === name) return action.newState;
    return state;
  };
};
/**
 * Format a model's actions
 *
 * @param {string} name
 * @param {class} Model
 * @param {object} newModels
 * @param {function} newDispatch
 * @param {function} realDispatch
 */


var createActions = function createActions(name, Model, newModels, newDispatch, realDispatch) {
  var model__proto__ = Model.prototype;
  var model = newModels[name];

  var setState = function setState(payload) {
    if (createActions.current.name !== name) {
      console.error(ERR.NO_SET_STATE(name));
      return;
    }

    var newState = _objectSpread({}, model.state, {}, payload);

    model.state = newState;
    return realDispatch({
      type: "".concat(name, "/").concat(createActions.current.actionName),
      newState: newState
    });
  };

  var setLoading = function setLoading(actionName, type, status) {
    model__proto__[actionName].loading = status;
    return realDispatch({
      type: "".concat(name, "/").concat(actionName, "/").concat(type, "_LOADING"),
      newState: _objectSpread({}, model.state, {
        loading: _defineProperty({}, actionName, status)
      })
    });
  };

  model__proto__.models = newModels;
  model__proto__.setState = setState;
  var skipList = ['constructor', 'models', 'setState'];
  newDispatch[name] = {};
  Object.getOwnPropertyNames(model__proto__).forEach(function (actionName) {
    if (skipList.includes(actionName)) return;
    var oldAction = model__proto__[actionName].bind(model);

    var newAction = function newAction() {
      var newCurrent = {
        name: name,
        actionName: actionName
      };
      var prevCurrent = createActions.current || newCurrent;
      createActions.current = newCurrent;
      var result = oldAction.apply(void 0, arguments);

      if (!isPromise(result)) {
        createActions.current = prevCurrent;
        return result;
      }

      setLoading(actionName, 'START', true);
      return new Promise(function (resolve, reject) {
        result.then(resolve).catch(reject).finally(function () {
          setLoading(actionName, 'STOP', false);
          createActions.current = prevCurrent;
        });
      });
    };

    newDispatch[name][actionName] = newAction;
    model__proto__[actionName] = newAction;
  });
};
/**
 * Initialize all models
 *
 * @param {object} models
 * @param {array} middleware
 * @return {object} Store
 */


exports.setStore = function setStore() {
  var models = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var middleware = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  var __DEV__ = process.env.NODE_ENV !== 'production';

  if (__DEV__) {
    if (!isObject(models)) throw new Error(ERR.NOT_OBJECT('models'));
    if (!Array.isArray(middleware)) throw new Error(ERR.NOT_ARRAY('middleware'));
  }

  var newModels = {};
  var newReducers = {};
  var nameModelList = Object.entries(models); // Reducers

  if (__DEV__) {
    nameModelList.forEach(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          name = _ref2[0],
          Model = _ref2[1];

      if (typeof Model !== 'function') throw new Error(ERR.NOT_CLASS('Model'));
      createReducer(name, Model, newModels, newReducers);
    });
  } else {
    nameModelList.forEach(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          name = _ref4[0],
          Model = _ref4[1];

      createReducer(name, Model, newModels, newReducers);
    });
  } // Store


  var __COMPOSE__ = window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__;
  var newCompose = __DEV__ && __COMPOSE__ ? __COMPOSE__ : redux.compose;
  var store = redux.createStore(redux.combineReducers(newReducers), newCompose(redux.applyMiddleware.apply(void 0, _toConsumableArray(middleware))));
  var realDispatch = store.dispatch;

  var newDispatch = function newDispatch() {
    console.error(ERR.NO_DISPATCH());
  };

  store.dispatch = newDispatch; // Actions

  nameModelList.forEach(function (_ref5) {
    var _ref6 = _slicedToArray(_ref5, 2),
        name = _ref6[0],
        Model = _ref6[1];

    createActions(name, Model, newModels, newDispatch, realDispatch);
  }); // store.add

  store.add = function (modelsToAdd) {
    if (__DEV__) {
      if (!isObject(modelsToAdd)) throw new Error(ERR.NOT_OBJECT('models'));
      Object.entries(modelsToAdd).forEach(function (_ref7) {
        var _ref8 = _slicedToArray(_ref7, 2),
            name = _ref8[0],
            Model = _ref8[1];

        if (name in newModels) return;
        if (typeof Model !== 'function') throw new Error(ERR.NOT_CLASS('Model'));
        createReducer(name, Model, newModels, newReducers);
        store.replaceReducer(redux.combineReducers(newReducers));
        createActions(name, Model, newModels, newDispatch, realDispatch);
      });
    } else {
      Object.entries(modelsToAdd).forEach(function (_ref9) {
        var _ref10 = _slicedToArray(_ref9, 2),
            name = _ref10[0],
            Model = _ref10[1];

        if (name in newModels) return;
        createReducer(name, Model, newModels, newReducers);
        store.replaceReducer(redux.combineReducers(newReducers));
        createActions(name, Model, newModels, newDispatch, realDispatch);
      });
    }
  };

  return store;
};
/**
 * Eject some models to a component
 *
 * @param {string[]} names
 * @return {function} Higher-order component
 */


exports.withStore = function withStore() {
  for (var _len = arguments.length, names = new Array(_len), _key = 0; _key < _len; _key++) {
    names[_key] = arguments[_key];
  }

  var __DEV__ = process.env.NODE_ENV !== 'production';

  var first = names[0];

  if (typeof first === 'string') {
    var mapStore = function mapStore(store) {
      return names.reduce(function (props, name) {
        return _objectSpread({}, props, {}, store[name]);
      }, {});
    };

    var mapState;

    if (__DEV__) {
      mapState = function mapState(state) {
        return names.reduce(function (props, name) {
          if (typeof name !== 'string') throw new Error(ERR.NOT_STRING('name'));
          var obj = state[name];
          if (typeof obj === 'undefined') throw new Error(ERR.NOT_EXIST(name));
          return _objectSpread({}, props, {}, obj);
        }, {});
      };
    } else {
      mapState = mapStore;
    }

    var mapDispatch = mapStore;
    return reactRedux.connect(mapState, mapDispatch);
  }

  if (isObject(first)) {
    var firstFiltered = {};

    var getProps = function getProps(props, name, keys, obj) {
      var modelProps = keys.reduce(function (part, key) {
        var val = obj[key];
        if (typeof val !== 'undefined') return _objectSpread({}, part, _defineProperty({}, key, val));
        firstFiltered[name].push(key);
        return part;
      }, {});
      return _objectSpread({}, props, {}, modelProps, {
        loading: obj.loading
      });
    };

    var _mapState;

    if (__DEV__) {
      _mapState = function _mapState(state) {
        return Object.entries(first).reduce(function (props, _ref11) {
          var _ref12 = _slicedToArray(_ref11, 2),
              name = _ref12[0],
              keys = _ref12[1];

          firstFiltered[name] = [];
          var obj = state[name];
          if (typeof obj === 'undefined') throw new Error(ERR.NOT_EXIST(name));
          return getProps(props, name, keys, obj);
        }, {});
      };
    } else {
      _mapState = function _mapState(state) {
        return Object.entries(first).reduce(function (props, _ref13) {
          var _ref14 = _slicedToArray(_ref13, 2),
              name = _ref14[0],
              keys = _ref14[1];

          firstFiltered[name] = [];
          return getProps(props, name, keys, state[name]);
        }, {});
      };
    }

    var _mapDispatch = function _mapDispatch(dispatch) {
      return Object.entries(firstFiltered).reduce(function (props, _ref15) {
        var _ref16 = _slicedToArray(_ref15, 2),
            name = _ref16[0],
            keys = _ref16[1];

        var obj = dispatch[name];
        var modelProps = keys.reduce(function (part, key) {
          return _objectSpread({}, part, _defineProperty({}, key, obj[key]));
        }, {});
        return _objectSpread({}, props, {}, modelProps);
      }, {});
    };

    return reactRedux.connect(_mapState, _mapDispatch);
  }

  return reactRedux.connect.apply(void 0, names);
};

Object.defineProperty(exports, 'Provider', {
  enumerable: true,
  get: function () {
    return reactRedux.Provider;
  }
});
Object.defineProperty(exports, 'batch', {
  enumerable: true,
  get: function () {
    return reactRedux.batch;
  }
});
